@@ -7,6 +7,12 @@ use core::marker::PhantomData;
 use core::ops::RangeInclusive;
 use core::prelude::v1::*;
 
+#[cfg(feature = "serde1")]
+use serde::{
+    de::{Deserialize, Deserializer, SeqAccess, Visitor},
+    ser::{Serialize, Serializer},
+};
+
 /// A map whose keys are stored as ranges bounded
 /// inclusively below and above `(start..=end)`.
 ///
@@ -28,7 +34,7 @@ use core::prelude::v1::*;
 pub struct RangeInclusiveMap<K, V, StepFnsT = K> {
     // Wrap ranges so that they are `Ord`.
     // See `range_wrapper.rs` for explanation.
-    btm: BTreeMap<RangeInclusiveStartWrapper<K>, V>,
+    pub(crate) btm: BTreeMap<RangeInclusiveStartWrapper<K>, V>,
     _phantom: PhantomData<StepFnsT>,
 }
 
@@ -533,6 +539,77 @@ where
     }
 }
 
+#[cfg(feature = "serde1")]
+impl<K, V> Serialize for RangeInclusiveMap<K, V>
+where
+    K: Ord + Clone + StepLite + Serialize,
+    V: Eq + Clone + Serialize,
+{
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        use serde::ser::SerializeSeq;
+        let mut seq = serializer.serialize_seq(Some(self.btm.len()))?;
+        for (k, v) in self.iter() {
+            seq.serialize_element(&((k.start(), k.end()), &v))?;
+        }
+        seq.end()
+    }
+}
+
+#[cfg(feature = "serde1")]
+impl<'de, K, V> Deserialize<'de> for RangeInclusiveMap<K, V>
+where
+    K: Ord + Clone + StepLite + Deserialize<'de>,
+    V: Eq + Clone + Deserialize<'de>,
+{
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        deserializer.deserialize_seq(RangeInclusiveMapVisitor::new())
+    }
+}
+
+#[cfg(feature = "serde1")]
+struct RangeInclusiveMapVisitor<K, V> {
+    marker: PhantomData<fn() -> RangeInclusiveMap<K, V>>,
+}
+
+#[cfg(feature = "serde1")]
+impl<K, V> RangeInclusiveMapVisitor<K, V> {
+    fn new() -> Self {
+        RangeInclusiveMapVisitor {
+            marker: PhantomData,
+        }
+    }
+}
+
+#[cfg(feature = "serde1")]
+impl<'de, K, V> Visitor<'de> for RangeInclusiveMapVisitor<K, V>
+where
+    K: Ord + Clone + StepLite + Deserialize<'de>,
+    V: Eq + Clone + Deserialize<'de>,
+{
+    type Value = RangeInclusiveMap<K, V>;
+
+    fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+        formatter.write_str("RangeInclusiveMap")
+    }
+
+    fn visit_seq<A>(self, mut access: A) -> Result<Self::Value, A::Error>
+    where
+        A: SeqAccess<'de>,
+    {
+        let mut range_inclusive_map = RangeInclusiveMap::new();
+        while let Some(((start, end), value)) = access.next_element()? {
+            range_inclusive_map.insert(start..=end, value);
+        }
+        Ok(range_inclusive_map)
+    }
+}
+
 pub struct Gaps<'a, K, V, StepFnsT> {
     /// Would be redundant, but we need an extra flag to
     /// avoid overflowing when dealing with inclusive ranges.
@@ -1243,4 +1320,32 @@ mod tests {
         map.insert(10..=11, ());
         assert_eq!(format!("{:?}", map), "{2..=5: (), 7..=8: (), 10..=11: ()}");
     }
+
+    // impl Serialize
+
+    #[cfg(feature = "serde1")]
+    #[test]
+    fn serialization() {
+        let mut range_map: RangeInclusiveMap<u32, bool> = RangeInclusiveMap::new();
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◆---◆ ◌ ◌ ◌ ◌ ◌ ◌
+        range_map.insert(1..=3, false);
+        // 0 1 2 3 4 5 6 7 8 9
+        // ◌ ◌ ◌ ◌ ◌ ◆---◆ ◌ ◌
+        range_map.insert(5..=7, true);
+        let output = serde_json::to_string(&range_map).expect("Failed to serialize");
+        assert_eq!(output, "[[[1,3],false],[[5,7],true]]");
+    }
+
+    // impl Deserialize
+
+    #[cfg(feature = "serde1")]
+    #[test]
+    fn deserialization() {
+        let input = "[[[1,3],false],[[5,7],true]]";
+        let range_map: RangeInclusiveMap<u32, bool> =
+            serde_json::from_str(input).expect("Failed to deserialize");
+        let reserialized = serde_json::to_string(&range_map).expect("Failed to re-serialize");
+        assert_eq!(reserialized, input);
+    }
 }
